#include <M5Stack.h>
#include <WiFi.h>
#include <Wire.h>
#include "time.h"

String lip;


const int motorL = 0x60;
const int motorR = 0x65;

byte speedL = 0x25;
byte speedR = 0x25;

int dur1=1400;
int dur2=500;
int prevd=0;
int prevs=0;

WiFiServer server(80);

const char* ssid       = "**********";
const char* password   = "**********";

const char* ntpServer =  "ntp.jst.mfeed.ad.jp";
const long  gmtOffset_sec = 9 * 3600; 
const int   daylightOffset_sec = 0;

void move(int motor, byte vset, byte data1){
  int vdata=vset << 2 | data1;
  Wire.beginTransmission(motor);
  Wire.write(0x00);
  Wire.write(vdata);
  Wire.endTransmission(true);
}

void ipPrint(){
  M5.Lcd.fillRect(20,20,320,20,TFT_BLACK);
  M5.Lcd.setCursor(20,20);
  M5.Lcd.print(lip);
}

void datePrint() {
  struct tm timeInfo;
  if (!getLocalTime(&timeInfo)) {
    Serial.println("Failed to obtain time");
  }
  if (prevd != (int)timeInfo.tm_mday) {
    M5.Lcd.fillRect(20,60,140,20,TFT_BLACK);
    M5.Lcd.setCursor(20,60);
    M5.Lcd.printf("%04d/%02d/%02d",timeInfo.tm_year + 1900, timeInfo.tm_mon + 1, timeInfo.tm_mday);
    prevd=(int)timeInfo.tm_mday;
  }
  if (prevs != (int)timeInfo.tm_sec){
    M5.Lcd.fillRect(140,60,140,20,TFT_BLACK);
    M5.Lcd.setCursor(140,60);
    M5.Lcd.printf("-%02d:%02d:%02d",timeInfo.tm_hour, timeInfo.tm_min, timeInfo.tm_sec);
    prevs=(int)timeInfo.tm_sec;
  }
}

void modePrint(String str){
  M5.Lcd.fillRect(20,100,320,20,TFT_BLACK);
  M5.Lcd.setCursor(20,100);
  M5.Lcd.print(str);
}

void setup() {
  Serial.begin(115200);
  M5.begin();
  Wire.begin();
  M5.Lcd.fillScreen(TFT_BLACK);
  M5.Lcd.setTextColor(TFT_GREEN);  // Adding a background colour erases previous text automatically
  M5.Lcd.setTextSize(2);

  // connect to WiFi
  Serial.printf("Connecting to %s ", ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  lip=WiFi.localIP().toString();
  Serial.println(" CONNECTED"+lip);
  ipPrint();
  // Set ntp time to local
  configTime(9 * 3600, 0, ntpServer);
 
  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  datePrint();
  
  server.begin();
  move(motorL,0,0);
  move(motorR,0,0);
  modePrint("STOP");
  delay(dur2);
}

void loop() {
  WiFiClient client = server.available();
  datePrint();
  if (client) {
    Serial.println("new client");
    String currentLine = "";
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        Serial.write(c);
        if (c == '\n') {
          if (currentLine.length() == 0) {
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println();
            client.println("<!DOCTYPE html>");
            client.println("<html>");
            client.println("<head>");
            client.println("<meta name='viewport' content='initial-scale=1.5'>");
            client.println("</head>");
            client.println("<body>");
            client.println("<form method='get'>");
            client.println("<font size='4'>M5 Mini-Car<br>");
            client.println("<input type='submit' name=0 value='ADVANCE' style='background-color:#88ff88; color:blue;'>");
            client.println("<br><br>");
            client.println("<input type='submit' name=1 value='BACK' style='background-color:#88ff88; color:blue;'>");
            client.println("<br><br>");
            client.println("<input type='submit' name=2 value='LEFT' style='background-color:#88ff88; color:blue;'>");
            client.println("<br><br>");
            client.println("<input type='submit' name=3 value='RIGHT' style='background-color:#88ff88; color:blue;'>");                  
            client.println("</form>");
            client.println("</body>");
            client.println("</html>");
            client.println();
            break;
          } else {
            currentLine = "";
          }
        } else if (c != '\r') {
          currentLine += c;
        }
 
        if (currentLine.endsWith("GET /?0=ADVANCE")) {
          move(motorL,speedL,2);
          move(motorR,speedR,2);        
          modePrint("ADVANCE");
          delay(dur1);
          move(motorL,0,0);
          move(motorR,0,0);  
          modePrint("STOP");       
        }
        if (currentLine.endsWith("GET /?1=BACK")) {
          move(motorL,speedL,1);
          move(motorR,speedR,1);    
          modePrint("BACK");
          delay(dur1);
          move(motorL,0,0);
          move(motorR,0,0);  
          modePrint("STOP");
        }
        if (currentLine.endsWith("GET /?2=LEFT")) {
          move(motorL,speedL,1);
          move(motorR,speedR,2);   
          modePrint("LEFT"); 
          delay(dur1);
          move(motorL,0,0);
          move(motorR,0,0);
          modePrint("STOP");      
        }
        if (currentLine.endsWith("GET /?3=RIGHT")) {
          move(motorL,speedL,2);
          move(motorR,speedR,1); 
          modePrint("RIGHT");
          delay(dur1);
          move(motorL,0,0);
          move(motorR,0,0);
          modePrint("STOP"); 
        }
      }
    }
    delay(dur2);
    client.stop();
    Serial.println("client disonnected");
  }
}
